/*
 * routines for keeping track of all NICs ever seen
 */

#include "libfma.h"
#include "lf_fabric.h"

#include "fma.h"
#include "fma_fabric.h"
#include "fma_map.h"

struct fma_nic_cache *
fma_add_nic_to_cache(
  struct lf_nic *nicp)
{
  struct fma_map_info *mip;
  struct fma_nic_cache *ncp;

  mip = A.map_info;

  /* Grow the cache array if needed */
  if (mip->size_nic_cache <= mip->num_nic_cache) {
     mip->size_nic_cache += 256;
    mip->nic_cache = (struct fma_nic_cache *) realloc(mip->nic_cache,
	sizeof(struct fma_nic_cache) * mip->size_nic_cache);
    if (mip->nic_cache == NULL) {
      LF_ERROR(("Error growing NIC cache"));
    }
  }

  /* Fill in this entry */
  ncp = mip->nic_cache + mip->num_nic_cache;
  LF_MAC_COPY(ncp->nc_mac_addr, nicp->mac_addr);
  LF_STRCPY(ncp->nc_hostname, nicp->host->hostname);
  ncp->nc_unique_id =  nicp->n_unique_id;
  FMA_NIC(nicp)->cache_index = mip->num_nic_cache;;

  /* update count of entries */
  ++mip->num_nic_cache;

  return ncp;

 except:
  fma_perror_exit(1);
  return NULL;
}

/*
 * Return pointer to NIC cache entry if found, else NULL
 */
struct fma_nic_cache *
fms_nic_cache_lookup(
  lf_mac_addr_t mac_addr)
{
  struct fma_map_info *mip;
  int n;

  mip = A.map_info;

  for (n=0; n<mip->num_nic_cache; ++n) {
    if (LF_MAC_CMP(mip->nic_cache[n].nc_mac_addr, mac_addr) == 0) {
      return &mip->nic_cache[n];
    }
  }

  return NULL;
}

/*
 * Look for the entry for a NIC and update it if found, else create
 * a brand new entry.
 */
struct fma_nic_cache *
fma_update_nic_cache(
  struct lf_nic *nicp)
{
  struct fma_nic_cache *ncp;

  /* Find cache entry for this NIC */
  ncp = fms_nic_cache_lookup(nicp->mac_addr);

  /* If no cache entry, create one */
  if (ncp == NULL) {
    ncp = fma_add_nic_to_cache(nicp);

  /* cache entry found, update the hostname if different */
  } else {
    if (strcmp(nicp->host->hostname, ncp->nc_hostname) != 0) {
      if (A.debug) {
	fma_log("NIC " LF_MAC_FORMAT " changed hostname from %s to %s",
	    LF_MAC_ARGS(nicp->mac_addr), nicp->mac_addr, nicp->host->hostname,
	    ncp->nc_hostname);
      }
      LF_STRCPY(ncp->nc_hostname, nicp->host->hostname);
    }
  }

  /* save ptr to cache entry */
  FMA_NIC(nicp)->cache_index = ncp - A.map_info->nic_cache;

  return ncp;
}
