#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include "myriexpress.h"
#include "mx_raw.h"


void
check_error(int cond_good, char *fmt, ...)
{
  va_list args;

  if (!(cond_good)) {
    va_start(args, fmt);
    vfprintf(stderr, fmt, args);
    va_end(args);
    exit(1);
  }
}

#define MAX_NODES 16384

uint64_t nic_id[MAX_NODES];
uint64_t gw_id[MAX_NODES];

int
main(int argc, char *argv[])
{
  uint32_t line, cnt, i, mac0, mac1, mac2, mac3, mac4, mac5;
  unsigned gw0, gw1, gw2, gw3, gw4, gw5;
  char string[256];
  int errno, rc;
  mx_raw_endpoint_t ep;
  FILE *file;
  
  /* open file */
  check_error(argc > 1, "MAC addresses file name missing\n");
  file = fopen(argv[1], "r");
  check_error(file != NULL, "Error opening config file %s: %d\n", 
	      argv[1], errno);
  
  /* init mx */
  mx_init();
  rc = mx_raw_open_endpoint(0, NULL, 0, &ep);
  check_error(!rc, "Error opening raw endpoint: %s\n", mx_strerror(rc));
  
  line = 0;
  cnt = 0;
  
  /* read the file */
  while (fgets(string, sizeof(string), file))
    {
      int mag;
      line++;
      rc = sscanf(string, "%x:%x:%x:%x:%x:%x %x:%x:%x:%x:%x:%x", 
		  &mac0, &mac1, &mac2, &mac3, &mac4, &mac5,
		  &gw0, &gw1, &gw2, &gw3, &gw4, &gw5);
      check_error(rc == 6 || rc == 12, "Bad MAC address(es) at line %d: %s\n", line, string);

      
      nic_id[cnt] = (((uint64_t) mac0 << 40) + ((uint64_t) mac1 << 32) 
		       + (mac2 << 24) + (mac3 << 16) + (mac4 << 8) + mac5);
      if (rc == 12)
	gw_id[cnt] = (((uint64_t) gw0 << 40) + ((uint64_t) gw1 << 32) 
		      + (gw2 << 24) + (gw3 << 16) + (gw4 << 8) + gw5);
      cnt += 1;
    }
  
  rc = mx_raw_set_route_begin(ep);
  check_error(!rc, "Error add peer init: %s\n", mx_strerror(rc));
  
  /* add the peers */
  for (i = 0; i < cnt; i++)
    {
      uint8_t route[6];
      int route_len;
      int node_type;

      if (gw_id[i]) {
	int j;
	for (j=0;j<6;j++)
	  route[5 - j] = gw_id[i] >> (j * 8);
	route_len = 6;
	node_type = MX_HOST_MXvM;
      } else {
	route_len = 0;
	node_type = MX_HOST_MX;
      }
	
      rc = mx_raw_set_route(ep, nic_id[i], route, route_len, 0, 0, node_type);
      check_error(!rc, "Error adding peer: %s\n", mx_strerror(rc));
    }

  rc = mx_raw_set_route_end(ep);
  check_error(!rc, "Error add peer init: %s\n", mx_strerror(rc));
  return 0;
}
